@@ -5,7 +5,6 @@
  * Author: Jonathan Blandford <jrb@redhat.com>
  * Based on code by: Federico Mena (Quartic), Szekeres Istvan (Pista)
 
-
 Ghostview interface to ghostscript
 
 When the GHOSTVIEW environment variable is set, ghostscript draws on
@@ -105,10 +104,7 @@ continuing.
 
 The DONE message indicates that ghostscript has finished processing.
 
-
-
- */
-
+*/
 
 #include "config.h"
 #include <string.h>
@@ -136,14 +132,15 @@ The DONE message indicates that ghostscript has finished processing.
 #endif
 #endif
 
-
-
-
 #define SIGVAL void
 
 #define RES 75
-typedef void (* GtkGSSignal0) (GtkObject *object, gpointer data);
-enum { FOO, LAST_SIGNAL };
+
+typedef void (*InterpreterMessageSignal)(GtkObject *object, gchar *msg,
+                                         gpointer data);
+
+enum { INTERPRETER_MESSAGE, LAST_SIGNAL };
+
 static Boolean broken_pipe = False;
 
 static SIGVAL catchPipe(int i)
@@ -156,8 +153,6 @@ static SIGVAL catchPipe(int i)
 #endif
 }
 
-
-
 /* Forward declarations */
 static void gtk_gs_init (GtkGS *gs);
 static void gtk_gs_class_init (GtkGSClass *klass);
@@ -167,7 +162,7 @@ static void gtk_gs_size_request (GtkWidget *widget, GtkRequisition *requisition)
 static void gtk_gs_size_allocate (GtkWidget *widget, GtkAllocation *allocation);
 static gint gtk_gs_widget_event(GtkWidget *widget, GdkEvent *event, gpointer data);
 static void gtk_gs_value_adjustment_changed (GtkAdjustment *adjustment, gpointer data);
-static void gtk_gs_marshal_signal_0 (GtkObject *object, GtkSignalFunc func, gpointer func_data, GtkArg *args);
+static void gtk_gs_interpreter_message(GtkGS *gs, gchar *msg, gpointer user_data);
 static void send_ps(GtkGS *gs, long begin, unsigned int len, gboolean close);
 static void set_up_page (GtkGS *gs);
 static void close_pipe (int p[2]);
@@ -177,18 +172,27 @@ static float compute_ydpi();
 static gboolean compute_size(GtkGS *gs);
 static void output (gpointer data, gint source, GdkInputCondition condition);
 static void input (gpointer data, gint source, GdkInputCondition condition);
-
-
 static void stop_interpreter (GtkGS *gs);
 static gint start_interpreter (GtkGS *gs);
 gboolean computeSize(void);
 
-
 static GtkWidgetClass *parent_class;
+
 static GtkGSClass *gs_class;
-static gint ghostview_signals[LAST_SIGNAL] = { 0 };
+
+static gint gtk_gs_signals[LAST_SIGNAL] = { 0 };
 
 /* Static, private functions */
+static void
+gtk_gs_marshaller(GtkObject *object, GtkSignalFunc func,
+                  gpointer func_data, GtkArg *args)
+{
+        InterpreterMessageSignal rfunc;
+
+        rfunc = (InterpreterMessageSignal)func;
+
+        (*rfunc)(object, GTK_VALUE_POINTER (args[0]), func_data);
+}
 
 static void
 gtk_gs_init (GtkGS *gs)
@@ -203,7 +207,7 @@ gtk_gs_init (GtkGS *gs)
         gs->height = -1;
         gs->busy      = FALSE;
         gs->changed   = FALSE;
-        gs->gs_scan_pdf_cmd = "gs -dNODISPLAY -dQUIET -sPDFname=%s -sDSCname=%s pdf2dsc.ps -c quit";
+        gs->gs_scan_pdf_cmd = GS_PATH " -dNODISPLAY -dQUIET -sPDFname=%s -sDSCname=%s pdf2dsc.ps -c quit";
         gs->gs_uncompress_cmd = "gzip -cd %s > %s";
         gs->gs_scanstyle = 0;
 	gs->gs_filename_raw = 0;
@@ -227,7 +231,6 @@ gtk_gs_init (GtkGS *gs)
 	gs->interpreter_output_id = 0;
 	gs->interpreter_error_id = 0;
 
-
         gs->ps_input = NULL;
         gs->input_buffer = NULL;
         gs->input_buffer_ptr = NULL;
@@ -264,11 +267,18 @@ gtk_gs_class_init (GtkGSClass *klass)
         parent_class    = gtk_type_class(gtk_widget_get_type());
         gs_class = klass;
 
+	gtk_gs_signals[INTERPRETER_MESSAGE] = gtk_signal_new(
+                       "interpreter_message", GTK_RUN_LAST, object_class->type,
+                       GTK_SIGNAL_OFFSET(GtkGSClass, interpreter_message),
+                       gtk_gs_marshaller, GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
+        gtk_object_class_add_signals(object_class, gtk_gs_signals, LAST_SIGNAL);
+
         object_class->destroy = gtk_gs_destroy;
 
         widget_class->realize       = gtk_gs_realize;
         widget_class->size_request  = gtk_gs_size_request;
         widget_class->size_allocate = gtk_gs_size_allocate;
+
         /* Create atoms */
         klass->gs_atom        = gdk_atom_intern ("GHOSTVIEW", FALSE);
         klass->gs_colors_atom = gdk_atom_intern ("GHOSTVIEW_COLORS", FALSE);
@@ -276,6 +286,17 @@ gtk_gs_class_init (GtkGSClass *klass)
         klass->page_atom      = gdk_atom_intern ("PAGE", FALSE);
         klass->done_atom      = gdk_atom_intern ("DONE", FALSE);
         klass->string_atom    = gdk_atom_intern ("STRING", FALSE);
+
+        /* a default handler for "interpreter_message" signal */
+        klass->interpreter_message = gtk_gs_interpreter_message;
+}
+
+/* free message as it was allocated in output() */
+static void
+gtk_gs_interpreter_message(GtkGS *gs, gchar *msg, gpointer user_data)
+{
+        g_print("interpreter message: %s\n", msg);
+        g_free(msg);
 }
 
 /* Clean all memory and temporal files */
@@ -293,7 +314,6 @@ gtk_gs_cleanup (GtkGS *gs)
                 g_free(gs->gs_filename_raw);
         if (gs->doc)
                 psfree(gs->doc);
-        
         if (gs->gs_filename_dsc){
                 /*
                 g_print("Removing dsc temp file: %s\n", gs->gs_filename_dsc);
@@ -301,7 +321,6 @@ gtk_gs_cleanup (GtkGS *gs)
                 unlink(gs->gs_filename_dsc);
                 gs->gs_filename_dsc = NULL;
         }
-        
         if (gs->gs_filename_unc){
                 /*
                 g_print("Removing unc temp file: %s\n", gs->gs_filename_unc);
@@ -313,8 +332,6 @@ gtk_gs_cleanup (GtkGS *gs)
                 g_free(gs->pages_marked);
                 gs->pages_marked = NULL;
         }
-                
-
         gs->doc = NULL;
   
 }
@@ -431,8 +448,10 @@ gtk_gs_realize (GtkWidget *widget)
                                   gs->pstarget,
                                   GTK_STATE_ACTIVE);
 
-        gs->width = attributes.width;
-        gs->height = attributes.height;
+        /*
+          gs->width = attributes.width;
+          gs->height = attributes.height;
+        */
 	/* XXX ??? */
         if ((gs->width > 0) && (gs->height > 0) && GTK_WIDGET_REALIZED(gs)) {
 		gtk_gs_munge_adjustments (gs);
@@ -488,7 +507,6 @@ gtk_gs_size_allocate (GtkWidget     *widget,
         if ((gs->width > 0) && (gs->height >0) && GTK_WIDGET_REALIZED(gs)) {
 		gtk_gs_munge_adjustments (gs);
         }
-
 }
 
 static gboolean 
@@ -548,18 +566,6 @@ gtk_gs_value_adjustment_changed (GtkAdjustment *adjustment, gpointer data)
         gdk_window_move (gs->pstarget, newx, newy);
 }
 
-static void
-gtk_gs_marshal_signal_0 (GtkObject     *object,
-                         GtkSignalFunc  func,
-                         gpointer       func_data,
-                         GtkArg        *args)
-{
-        /*        GtkGSSignal0 rfunc;*/
-        /*        rfunc = (GtkGSSignal0) func;*/
-        
-        /*        (* rfunc) (object, func_data);*/
-}
-
 void 
 gtk_gs_center_page(GtkGS *gs)
 {
@@ -694,7 +700,6 @@ set_up_page (GtkGS *gs)
                 At this point the old pixmap is still in the background. We should clean 
                 it up before we continue. 
                 */
-
         }
 
         pprivate = (GdkPixmapPrivate *) gs->bpixmap;
@@ -718,8 +723,6 @@ set_up_page (GtkGS *gs)
                             buf,
                             strlen(buf));
         gdk_flush(); 
-
-        //        gtk_gs_center_page (gs);
 }
 
 static void
@@ -757,7 +760,7 @@ interpreter_failed (GtkGS *gs)
 static void
 output (gpointer data, gint source, GdkInputCondition condition)
 {
-        char buf[MAX_BUFSIZE+1];
+        char buf[MAX_BUFSIZE+1], *msg;
         guint bytes = 0;
         GtkGS *gs = GTK_GS (data);
         
@@ -788,12 +791,9 @@ output (gpointer data, gint source, GdkInputCondition condition)
         }
         if (bytes > 0) {
                 buf[bytes] = '\0';
-                /* 
-                 * FIXME:  er, we actually need to do something with 
-                 * the message...
-                 */
-                g_print ("we got a message from ghostscript:\n");
-                g_print ("*****\n\t%s\n*****\n", buf);
+                msg = g_strdup(buf);
+                gtk_signal_emit(GTK_OBJECT(gs),
+                                gtk_gs_signals[INTERPRETER_MESSAGE], msg);
         }
 }
 
@@ -973,7 +973,7 @@ start_interpreter (GtkGS *gs)
                 
                 sprintf(buffer, "GHOSTVIEW=%ld",
                         ((GdkWindowPrivate*)(gs->pstarget))->xwindow);
-                
+
 		putenv(g_strdup(buffer));
                 
                 if (gs->gs_filename == NULL) {
@@ -1158,9 +1158,6 @@ gtk_gs_load (GtkGS *gs, char *fname)
         struct stat sbuf;
 	int i;
 	g_assert(gs != NULL);
-        /*
-        printf (" in gs_load\n");
-        */
 	if (fname == NULL) {
                 if(gdk_window_is_visible(gs->pstarget))
                         gdk_window_hide(gs->pstarget);
@@ -1227,7 +1224,6 @@ gtk_gs_load (GtkGS *gs, char *fname)
                         */
                         gs->structured_doc = FALSE; 
                         gs->send_filename_to_gs = TRUE;
-
                 }
 
                 /*
@@ -1236,6 +1232,7 @@ gtk_gs_load (GtkGS *gs, char *fname)
                 if ((!gs->doc->epsf && gs->doc->numpages>0) || 
                     (gs->doc->epsf && gs->doc->numpages>1)) {
                         gs->structured_doc = TRUE; 
+                        gs->send_filename_to_gs = FALSE;
                         gs->pages_marked = g_malloc(sizeof(gint) * gs->doc->numpages
 );
                         for (i=0;i<gs->doc->numpages; i++)
@@ -1307,6 +1304,7 @@ gboolean
 gtk_gs_goto_page (GtkGS *gs, gint page) 
 {
         struct stat sbuf;
+
 	g_assert (gs != NULL);
 
         /*
@@ -1321,10 +1319,13 @@ gtk_gs_goto_page (GtkGS *gs, gint page)
         
         if (gs->gs_psfile) {
         /* has it changed? */
-                if (!stat(gs->gs_filename, &sbuf) && gs->mtime != sbuf.st_mtime) {
+                if (!stat(gs->gs_filename, &sbuf) &&
+                    gs->mtime != sbuf.st_mtime) {
+                        gchar *fname = g_strdup(gs->gs_filename);
                         g_print ("file has changed!!!\n");
                         fclose(gs->gs_psfile);
-                        gtk_gs_load (gs, gs->gs_filename);
+                        gtk_gs_load (gs, fname);
+                        g_free(fname);
                 }
         }
 
@@ -1354,7 +1355,6 @@ gtk_gs_goto_page (GtkGS *gs, gint page)
                 gs->changed = FALSE;
                 gs->current_page = page;
                 
-
                 if (is_interpreter_ready (gs)) {
                         /*g_print ("the interpreter is ready!\n"); */
                         gtk_gs_next_page (gs);
@@ -1383,26 +1383,24 @@ gtk_gs_goto_page (GtkGS *gs, gint page)
                 g_print("sending page [non structured]\n");
 
                 if (is_interpreter_ready (gs)) {
-                        /* g_print ("the interpreter is ready!\n"); */
                 } else {
                         /* g_print ("the interpreter is not ready -- starting now\n"); */
                         gtk_gs_enable_interpreter (gs);
+
                         /*
-                        send_ps(gs, gs->doc->beginprolog, 
-                                gs->doc->lenprolog, FALSE);
-                        send_ps(gs, gs->doc->beginsetup, 
-                                gs->doc->lensetup, FALSE);
+                          send_ps(gs, gs->doc->beginprolog, 
+                          gs->doc->lenprolog, FALSE);
+                          send_ps(gs, gs->doc->beginsetup, 
+                          gs->doc->lensetup, FALSE);
                         */
                 }
-                gtk_gs_next_page (gs);
+                gtk_gs_next_page(gs);
         }
         return TRUE;
 }
 
 /*
  * set pagemedia sets the media from
- * if set_override is TRUE, then override_media is set to true.
- * if set_default is TRUE, then the pagemedia used is set to be the default one.
  * if new_pagemedia is -1, then it is set to either
  *  a) the default settings of pageid, if they exist, or if pageid != -1.
  *  b) the default setting of the document, if it exists.
@@ -1439,8 +1437,6 @@ gtk_gs_set_pagemedia(GtkGS *gs, gint new_pagemedia, gint pageid)
                 }
         }       
 
-        /*  g_print("page media %d\n", new_pagemedia); */
-        
         /* Compute bounding box */
         if (gs->doc && gs->doc->epsf 
             && (gs->doc->boundingbox[URX] > gs->doc->boundingbox[LLX]) &&
@@ -1464,17 +1460,16 @@ gtk_gs_set_pagemedia(GtkGS *gs, gint new_pagemedia, gint pageid)
         /* If bounding box changed, setup for new size. */
         if ((new_llx != gs->llx) || (new_lly != gs->lly) ||
             (new_urx != gs->urx) || (new_ury != gs->ury)) {
-                //                gtk_gs_disable_interpreter (gs);
+                /* gtk_gs_disable_interpreter (gs); */
                 gs->llx = new_llx;
                 gs->lly = new_lly;
                 gs->urx = new_urx;
                 gs->ury = new_ury;
                 changed = TRUE;
-                /* g_print ("setting up the page l: %d l:%d u:%d u:%d\n",new_llx,new_lly,new_urx,new_ury); */
 
-                set_up_page(gs);
+                if(GTK_WIDGET_REALIZED(gs))
+                        set_up_page(gs);
         }
-        /* g_print ("\tall done.\n"); */
 	if(changed) {
 		gtk_widget_queue_resize (&(gs->widget));
 	}
@@ -1492,7 +1487,8 @@ gtk_gs_set_orientation (GtkGS *gs, gint orientation) {
         if (gs->orientation != orientation) { 
                 gs->orientation = orientation;
                 gs->changed = TRUE;
-                set_up_page(gs);
+                if(GTK_WIDGET_REALIZED(gs))
+                        set_up_page(gs);
                 return TRUE;
         }
 	gtk_widget_queue_resize (&(gs->widget));
@@ -1503,8 +1499,6 @@ void
 gtk_gs_set_zoom (GtkGS *gs, gfloat zoom) {
 	g_assert (gs != NULL);
 
-        /* g_print ("zoom factor is %f\n",zoom); */
-        
         if (gs->zoom_factor != zoom) { 
                 gs->zoom_factor = zoom;
                 set_up_page(gs);
