@@ -9,6 +9,7 @@
 #include <gtk/gtkwidget.h>
 
 #include "gtkgs.h"
+#include "ggvwindow.h"
 
 extern gchar *gs_cmd;
 extern gchar *gs_scan_pdf_cmd;
@@ -17,12 +18,12 @@ extern gchar *gs_print_cmd;
 
 extern gint gs_def_media;
 extern gboolean gs_antialiased, gs_override_media;
-extern gboolean gs_panel, gs_menubar;
+extern gboolean gs_panel, gs_menubar, gs_auto_jump;
 extern gint gs_num_recent;
 
 void load_prefs(gchar *);
 void save_prefs(gchar *);
-void set_prefs (GtkGS *);
+void set_prefs (ggv_window *);
 
 #ifndef MAX_RECENT
 #define MAX_RECENT 6
